import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { scaleOrdinal } from 'd3-scale';
import { schemeSet1 } from 'd3-scale-chromatic';
import { produce } from 'immer';
import { useEffect, useMemo, useReducer, } from 'react';
import { useBBoxObserver } from 'react-d3-utils';
import { bboxContext } from '../contexts/bboxContext';
import { LegendProvider } from '../contexts/legendContext';
import { legendOffsetContext } from '../contexts/legendOffsetContext';
import { plotContext, plotDispatchContext, plotReducer, useAxisContext, } from '../contexts/plotContext';
import { usePlotEventsPlotContext, usePlotOverrides, } from '../contexts/plotController/plotControllerContext';
import { useId } from '../utils';
import { splitChildren } from '../utils/splitChildren';
import { usePlotSizes } from '../utils/usePlotSizes';
import Tracking from './Tracking';
import TransparentRect from './TransparentRect';
const reducerCurr = produce(plotReducer);
const initialState = {
    headingPosition: null,
    legendPosition: null,
    legendMargin: 0,
    series: [],
    axes: {},
};
const defaultSvgStyle = {
    overflow: 'visible',
    fontFamily: 'Arial, Helvetica, sans-serif',
    userSelect: 'none',
    // Without this, Safari on iOS still triggers selection from a long press.
    WebkitUserSelect: 'none',
};
const defaultEventSvgStyle = {
    // Prevent default touch behavior on the plot only when events are enabled.
    touchAction: 'none',
};
export function Plot(props) {
    const { width, height, colorScheme = schemeSet1, margin = {}, svgStyle = {}, svgId, svgClassName, plotViewportStyle = {}, seriesViewportStyle = {}, controllerId, children, } = props;
    const plotId = useId(undefined, 'plot');
    const [state, dispatch] = useReducer(reducerCurr, initialState, undefined);
    const { series, annotations, topAxis, rightAxis, bottomAxis, leftAxis, heading, legend, } = splitChildren(children);
    if (width === undefined) {
        throw new Error('width is mandatory');
    }
    if (height === undefined) {
        throw new Error('height is mandatory');
    }
    const plotEvents = usePlotEventsPlotContext({ controllerId });
    useEffect(() => {
        if (!plotEvents)
            return;
        plotEvents.registerPlot(plotId);
        return () => plotEvents.unregisterPlot(plotId);
    }, [plotEvents, plotId]);
    const plotOverrides = usePlotOverrides({ controllerId });
    // Bounding boxes used to adapt viewport size.
    const headingBbox = useBBoxObserver();
    const topAxisBbox = useBBoxObserver();
    const rightAxisBbox = useBBoxObserver();
    const bottomAxisBbox = useBBoxObserver();
    const leftAxisBbox = useBBoxObserver();
    const legendBbox = useBBoxObserver();
    // Distances in plot.
    const { plotWidth, plotHeight, topOffset, leftOffset, legendOffset } = usePlotSizes({
        width,
        height,
        margin,
        axes: state.axes,
        topAxisHeight: topAxisBbox.height,
        rightAxisWidth: rightAxisBbox.width,
        bottomAxisHeight: bottomAxisBbox.height,
        leftAxisWidth: leftAxisBbox.width,
        headingPosition: state.headingPosition,
        headingHeight: headingBbox.height,
        legendPosition: state.legendPosition,
        legendMargin: state.legendMargin,
        legendWidth: legendBbox.width,
        legendHeight: legendBbox.height,
    });
    // Set scales.
    const axisContext = useAxisContext(state, plotOverrides.axes, {
        plotWidth,
        plotHeight,
    });
    const ids = useMemo(() => state.series.map(({ id }) => id), [state.series]);
    const colorScaler = useMemo(() => scaleOrdinal().range(colorScheme).domain(ids), [colorScheme, ids]);
    const plotContextValue = useMemo(() => {
        return {
            width,
            height,
            plotWidth,
            plotHeight,
            axisContext,
            colorScaler,
        };
    }, [width, height, plotWidth, plotHeight, axisContext, colorScaler]);
    const finalSvgStyle = {
        ...defaultSvgStyle,
        ...(plotEvents ? defaultEventSvgStyle : null),
        ...svgStyle,
    };
    return (_jsx(plotContext.Provider, { value: plotContextValue, children: _jsx(plotDispatchContext.Provider, { value: dispatch, children: _jsx(LegendProvider, { children: _jsxs("div", { style: { position: 'relative', width: width, height: height }, children: [_jsxs("svg", { xmlns: "http://www.w3.org/2000/svg", width: width, height: height, style: {
                                ...finalSvgStyle,
                                position: 'absolute',
                                top: '0',
                                left: '0',
                            }, id: svgId, className: svgClassName, children: [_jsx(TransparentRect, { width: width, height: height, style: plotViewportStyle }), _jsxs("g", { transform: `translate(${leftOffset}, ${topOffset})`, children: [_jsx(TransparentRect, { width: plotWidth, height: plotHeight, style: seriesViewportStyle }), _jsx("clipPath", { id: `seriesViewportClip-${plotId}`, children: _jsx("rect", { width: plotWidth, height: plotHeight }) }), _jsx("g", { style: { clipPath: `url(#seriesViewportClip-${plotId})` }, children: series }), _jsx(bboxContext.Provider, { value: topAxisBbox.ref, children: _jsx("g", { children: topAxis }) }), _jsx(bboxContext.Provider, { value: rightAxisBbox.ref, children: _jsx("g", { children: rightAxis }) }), _jsx(bboxContext.Provider, { value: bottomAxisBbox.ref, children: _jsx("g", { children: bottomAxis }) }), _jsx(bboxContext.Provider, { value: leftAxisBbox.ref, children: _jsx("g", { children: leftAxis }) })] }), _jsx("g", { ref: headingBbox.ref, children: heading })] }), _jsxs("svg", { xmlns: "http://www.w3.org/2000/svg", width: width, height: height, style: {
                                ...finalSvgStyle,
                                position: 'absolute',
                                top: '0',
                                left: '0',
                            }, id: svgId ? `${svgId}-annotations` : undefined, className: svgClassName, children: [_jsxs("g", { transform: `translate(${leftOffset}, ${topOffset})`, children: [_jsx("g", { style: { clipPath: `url(#seriesViewportClip-${plotId})` }, children: annotations }), plotEvents ? (_jsx(Tracking, { plotId: plotId, plotEvents: plotEvents, stateSeries: state.series, axisContext: axisContext, plotWidth: plotWidth, plotHeight: plotHeight })) : null] }), _jsx("g", { transform: `translate(${leftOffset}, ${topOffset})`, children: _jsx(legendOffsetContext.Provider, { value: legendOffset, children: _jsx("g", { ref: legendBbox.ref, children: legend }) }) })] })] }) }) }) }));
}
//# sourceMappingURL=Plot.js.map